/* $Header: /usr/local/cvs/keyring/src/keyring.c,v 1.17 2002/12/14 00:11:40 BaeLang Exp $ */

#include <winsock2.h>
#include <stdio.h>
#include <string.h>
#include <tlhelp32.h>
#include <time.h>

#include "keyring.h"

static char* rcsid="$Id: keyring.c,v 1.17 2002/12/14 00:11:40 BaeLang Exp $";

extern unsigned short int enable_debug_privs();
extern unsigned short int find_offset(char *executable_path);

unsigned short int no_loop = FALSE;
unsigned short int verbose = FALSE;

void print_help(char * MyName) {
	printf("\n\tKeyring Version %s $Name: version2_2 $ $State: Exp $\n\n", KEYRING_VERSION);
	printf("\n\tOptions for %s:\n", MyName);
	printf("\n\t  -h  print output help information (but you knew that, didn't you.)");
	printf("\n\t  -f <path/to/config/file>  specifies the path to some config file");
	printf("\n\t  -v run in verbose mode (copious debug information)");
	printf("\n\t  -i <interval> specify the send interval in seconds.  ");
	printf("\n\t  -a <address> send-to ip address");
	printf("\n\t  -p <port> send-to udp port");
	printf("\n\t  -n <name> name of program to search for (i.e. eqgame or testeqgame)");
	printf("\n\t  -o <offset> key location");
	printf("\n\t  -l <executable_path> print key location (offset) and exit.");
	printf("\n\t  -e rune once and Exit.  (no looping)");
	printf("\n");
	}

/*
*  subroutine to read the 64 bit key from memory. 
*/
void readkey (HANDLE hProcess) {

	unsigned long addr;
	ULONGLONG key, lkey = 0xffffffffffffffff;
	/* 
	* some people have reported better luck with one of the two lines 
	* that are listed below, depending on your compiler.  
	* your mileage may vary.
	*/
	//unsigned long long key = 0xffffffffffffffff;
	//unsigned long long key = 0xffffffffffffffffULL;
	
	addr = config.SessionKeyLocation;

	/* 
	* read the memory.  loop through it unless the no_loop option was specified.
	*/
	do {

		if (ReadProcessMemory (hProcess, (void *)addr, &key, 8, NULL) == 0) {
			printf ("ReadProcessMemory on 8 bytes at 0x%08x failed: %u\n", addr, GetLastError());
			}
		else {
			/*
			* if the key changes, send the new one.
			*/
			if ( lkey != key ) {
#if defined (DEBUG)
				printf ("key:\t0x%016llx lkey:\t0x%016llx\n", key, lkey);
#endif
				if (SendSessionKey(key) != SOCKET_ERROR) {
					printf("Sent the key to %s:%d\n", config.seq_ip, config.seq_port);
					}
				else {
					printf("Failed to send the key to %s:%d\n", config.seq_ip, config.seq_port);
					}
				/*
				* after we send the new key, 
				* set the new old key value to the new key value.  
				* or somethign.
				*/
				lkey = key;
				}
			}  // end of the else

		/*
		* if the send interval is 0, just exit.
		* else, sleep for the number of seconds listed in the config file.
		*/
		if (config.SendInterval != 0) {
			Sleep(config.SendInterval*1000);
			/*
			* some compilers need the sleep call to be all lower case.
			*/
			//sleep(config.SendInterval*1000);
			}
		else {
			exit (0) ;
			}
		}  while (! no_loop) ; // end of the key read loop
	} // end of readkey


/*
* subroutine to scan the process list and find everquest (EQGame.exe)
*
* i have moved the process name to the config file, 
* since if you want to use test the offset is going to be different anyway.
*/
int scanproclist ( char* ProcessName ) {
    HANDLE         hProcessSnap = NULL;
    PROCESSENTRY32 pe32      = {0};

    //  Take a snapshot of all processes in the system.
    hProcessSnap = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);

    if (hProcessSnap == INVALID_HANDLE_VALUE) {
        return 0;
	   }

    //  Fill in the size of the structure before using it.
    pe32.dwSize = sizeof(PROCESSENTRY32);

    if (Process32First(hProcessSnap, &pe32)) {
		HANDLE hProcess;
        	do {
          	LPSTR pCurChar;
			char pName[512];
            	// strip path and leave exe filename
            	for (pCurChar = (pe32.szExeFile + strlen (pe32.szExeFile));
               *pCurChar != '\\' && pCurChar != pe32.szExeFile - 1;
               --pCurChar)

            	strcpy(pName, pCurChar);
			strlwr(pName);  // make it case insensitive

			if ( (strncmp (pName, ProcessName, strlen(ProcessName) ) == 0) ) {
				if (verbose) {
					printf ("found %s - pid = %u\n\n", ProcessName, pe32.th32ProcessID);
					}
				hProcess = OpenProcess (PROCESS_VM_READ, FALSE, pe32.th32ProcessID);
				/*
				* only read access is needed.
				* some people prefer full access. if that is you, 
				* use the line below instead.
				*/
				//hProcess = OpenProcess (PROCESS_ALL_ACCESS, FALSE, pe32.th32ProcessID);
				if (hProcess == NULL) {
					DWORD dw;
					dw = GetLastError();
					printf ("OpenProcess failed, error: %u\n", dw);
					return 0;
					}
				readkey (hProcess);
				}
	  		}
		while (Process32Next(hProcessSnap, &pe32));
		}

	CloseHandle (hProcessSnap);
	return 0;
	} //end of scanproclist


/*
* subroutine to read the config file 
* use the default config file defined at the top
* or use the config file passed in as a command line argument
*
* command line arguments override values in the config file. i never realized
* what a pain that is to code in.
*
*/

int ReadConfig (char* ConfFile) {

	char conf_buffer[CONF_SIZE];

	if (verbose) {
		printf("Readconfig is reading: %s \n", ConfFile) ;
		}

	/*
	* i had problems here until i limeted the program name to 16 characters.
	*/
	if (*options.ProgramName) {
		strcpy(config.ProgramName, options.ProgramName);
		if (verbose){printf("using command line ProgramName: %s\n",options.ProgramName);}
		}
	else {
		GetPrivateProfileString("Client", "ProgramName", PROGRAM_NAME, conf_buffer, CONF_SIZE, ConfFile);
		strcpy(config.ProgramName, conf_buffer);
		if (verbose){printf("using ini file ProgramName: %s\n",config.ProgramName);}
		}

	if (options.SessionKeyLocation) {
		config.SessionKeyLocation = options.SessionKeyLocation;
		}
	else {
		GetPrivateProfileString("Client", "SessionKeyLocation", OFFSET, conf_buffer, CONF_SIZE, ConfFile);
		config.SessionKeyLocation = strtol(conf_buffer,NULL,16);
		}

	if (options.SendInterval) {
		config.SendInterval = options.SendInterval;
		}
	else {
		GetPrivateProfileString("Client", "SendInterval", SEND_INTERVAL, conf_buffer, CONF_SIZE, ConfFile);
		config.SendInterval = atoi(conf_buffer);
		}

	if (*options.seq_ip) {
		strcpy(config.seq_ip, options.seq_ip) ;
		}
	else {
		GetPrivateProfileString("ShowEQ", "IP", ADDRESS, conf_buffer, CONF_SIZE, ConfFile);
		strcpy(config.seq_ip, conf_buffer);
		}

	if (options.seq_port) {
		config.seq_port = options.seq_port ;
		}
	else {
		GetPrivateProfileString("ShowEQ", "Port", PORT, conf_buffer, CONF_SIZE, ConfFile);
		config.seq_port = atoi(conf_buffer);
		}

	if (verbose) {
		printf("ProgramName is: %s\n", config.ProgramName ) ;
		printf("key location is: %8.8#lx \n", config.SessionKeyLocation ) ;
		printf("Send interval is: %i seconds.\n", config.SendInterval ) ;
		printf("destination IP is: %s:%u \n", config.seq_ip,config.seq_port ) ;
		}

	if (config.SessionKeyLocation > 0) {
		return (1);
		}
	else {
		return (0) ;
		}
	}  //end of ReadConfig


/*
* subroutine to send the session key, to the showEQ box
* via UDP on the port specified in the config file.
*/
int SendSessionKey(unsigned long long SessionKey) {
	int ret;
	WSADATA wsd;
	SOCKET ssocket;
	SOCKADDR_IN seq;

	if (verbose) {
		printf(" SendSessionKey function called with key: %ull", SendSessionKey);
		printf("to destination IP: %s:%u \n", config.seq_ip, config.seq_port ) ;
		}

	if (WSAStartup(MAKEWORD(2, 2), &wsd) != 0) {
		printf("WSAStartup failed!\n");
		return SOCKET_ERROR;
		}

	ssocket = socket(AF_INET, SOCK_DGRAM, 0);

	if (ssocket == INVALID_SOCKET) {
		printf("socket() failed; %d\n", WSAGetLastError());
		return SOCKET_ERROR;
		}

	seq.sin_family = AF_INET;
	seq.sin_port = htons((short)config.seq_port);
	seq.sin_addr.s_addr = inet_addr(config.seq_ip);

	ret = sendto(ssocket, (char *) &SessionKey, sizeof(ULONGLONG), 0, (SOCKADDR *)&seq, sizeof(seq));

	if (ret == SOCKET_ERROR) {
		return SOCKET_ERROR;
		}

	closesocket(ssocket);
	WSACleanup();

	return 0;
	}

/*
* the main program of course.
*
* works like this:
*  check for command line arguments
*  handle them if there are any
*  read the config file
*  scan the process list for everquest, loop until it is found.
*  read 64 bits of memory from the location specified by offset 
*    from the memory of the process ID returned from the process list scan
*    loop doing this at the interval specified in the config file
*   if the key changes, send this key to SEQ at the address:port 
*   as specified in the config file.
*  
*/
int main( int argc, char* argv[] ) {

	char * MyName = argv[0];

	options.ConfFile = CONF_FILE;

	/*
	* handle command line arguments here
	*
	* -h  print output help information 
	* -f <path/to/config/file>  specifies the path to alternate config file 
	* -v run in verbose mode (copious debug information)
	* -i <interval> specify the send interval in seconds.  
	* -a <address> send-to ip address
	* -p <port> send-to udp port
	* -n <name> name of program to search for (i.e. eqgame or testeqgame)
	* -o <offset> key location
	* -l <executible_path> find key location offset and exit
	* -e rune once and Exit.  (no looping)
	*/
	while ((argc > 1) && (argv[1][0] == '-')) {

		switch (argv[1][1]) {

			case 'h':
				print_help(MyName);
				exit(0);
				break;

			case 'f':
				options.ConfFile = strdup(argv[2]);
				++argv;
				--argc;
				break;

			case 'v':
				verbose = TRUE;
				printf ("\nMaximum verbosity.\n");
				break;

			case 'i':
				options.SendInterval = atoi(argv[2]);
				++argv;
				--argc;
				break;

			case 'a':
				strcpy (options.seq_ip, argv[2]);
				++argv;
				--argc;
				break;

			case 'p':
				options.seq_port = atoi(argv[2]);
				++argv;
				--argc;
				break;

			case 'n':
				strcpy (options.ProgramName, argv[2]);
				++argv;
				--argc;
				break;

			case 'o':
				options.SessionKeyLocation = strtol(argv[2],NULL,16);
				++argv;
				--argc;
				break;

			case 'l':
				if (argv[2]) {
					executable_path = argv[2];
					if (verbose) {
						printf("using executable path from command line %s\n", executable_path);
						}
					++argv;
					--argc;
					}
				else {
					executable_path = EQ_EXECUTABLE;
					if (verbose) {
						printf("using default executable path %s\n", executable_path);
						}
					}
				find_offset(executable_path);
				exit(0);
				break;

			case 'e':
				no_loop = TRUE;
				break;

			default:
				printf ("Some sort of error with your options, and i am too lazy to figure it out.\n");
				exit(1) ;
				break;
			} // end of switch
		++argv;
		--argc;
		} // end of while for arguments


#if defined (DEBUG)
	printf ("We are in debug mode! debug mode implies verbose mode.\n");
	verbose = TRUE;
#endif
	if (verbose) {
		printf ("main program starting. \n");
		printf ("%s based on keyring version %s.\n", MyName, KEYRING_VERSION);
		if (no_loop) {printf("no_loop mode selected.\n");}
		printf("config file name is: %s \n", options.ConfFile) ;
		}
	
	/* 
	* read configuration file.  the file must exist.  
	* exit if the file does not exist.
	* 
	* although, pretty much everything can be specified on the command line.
	* so i need to fix error handling and make it OK for the config file to
	* not exist.
	*/
	
	if (enable_debug_privs() == FALSE) {
		printf("unable to set debug privlidges. you may see the dreaded error 5.\n");
		}

	while (ReadConfig(options.ConfFile) == 1) {
		if (verbose) {
			printf ("scanning for %s\n", config.ProgramName);
			}
		scanproclist(config.ProgramName);
		if (no_loop) {
			exit(0) ;
			}
		else {
			Sleep (10000); /* pause 10 seconds between checks */
			}
		}
	printf("some sort of problem with the config file: %s \n", options.ConfFile) ;
	return (1) ;
	}




